use actix_web::{http::header::LOCATION, web, HttpResponse, Responder};
use chrono::NaiveTime;
use garde::Validate;
use sqlx::PgPool;

use crate::{
    endpoints::{
        availability::{find_adjacend_availability, post_new::AvailabillityForm},
        IdPath,
    },
    models::{Availability, AvailabilityChangeset, AvailabilityContext, AvailabilityTime, User},
    utils::{self, ApplicationError},
    END_OF_DAY,
};

#[actix_web::post("/availabillity/edit/{id}")]
pub async fn post(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
    form: web::Form<AvailabillityForm>,
) -> Result<impl Responder, ApplicationError> {
    let Some(availability) = Availability::read_by_id(pool.get_ref(), path.id).await? else {
        return Ok(HttpResponse::NotFound().finish());
    };

    if availability.user_id != user.id {
        return Err(ApplicationError::Unauthorized);
    }

    let existing_availabilities: Vec<Availability> =
        Availability::read_by_user_and_date(pool.get_ref(), user.id, &availability.date)
            .await?
            .into_iter()
            .filter(|a| a.id != availability.id)
            .collect();

    let context = AvailabilityContext {
        existing_availabilities: existing_availabilities.clone(),
    };

    let time = if form.from.is_some() && form.till.is_some() {
        let end_time = if form.till.unwrap() == NaiveTime::from_hms_opt(23, 59, 0).unwrap() {
            END_OF_DAY
        } else {
            form.till.unwrap()
        };
        AvailabilityTime::Temporarily(form.from.unwrap(), end_time)
    } else {
        AvailabilityTime::WholeDay
    };

    let mut changeset = AvailabilityChangeset {
        time,
        comment: form.comment.clone(),
    };

    if let Err(e) = changeset.validate_with(&context) {
        return Ok(HttpResponse::BadRequest().body(e.to_string()));
    };

    if let Some(a) =
        find_adjacend_availability(&changeset, Some(availability.id), &existing_availabilities)
    {
        let (changeset_start, changeset_end) = changeset.time.time_tuple_opt().unwrap();
        let (adjacent_start, adjacent_end) = changeset.time.time_tuple_opt().unwrap();

        if adjacent_end == changeset_start {
            changeset.time = AvailabilityTime::Temporarily(adjacent_start, changeset_end);
        }

        if adjacent_start == changeset_end {
            changeset.time = AvailabilityTime::Temporarily(changeset_start, adjacent_end);
        }

        Availability::update(pool.get_ref(), a.id, changeset).await?;
        Availability::delete(pool.get_ref(), availability.id).await?;
    } else {
        Availability::update(pool.get_ref(), availability.id, changeset).await?;
    }

    let url = utils::get_return_url_for_date(&form.date);
    Ok(HttpResponse::Found()
        .insert_header((LOCATION, url.clone()))
        .insert_header(("HX-LOCATION", url))
        .finish())
}
